from __future__ import absolute_import
#
# Global Cache network devices - core definitions
#
# Copyright 2010-2016,2019 Perry The Cynic. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import deque
import re
import sys
import socket
import struct
import time
from distutils.version import StrictVersion

import asyn
import asyn.resolve
import asyn.utility

from . import gcir

DEBUG = None


#
# Hardware rules and limits
#
PULSE_MIN = 80E-6			# minimum allowed time of an IR pulse leg (seconds)
PULSE_MIN_FLEX = 5			# experimental floor for iTach Flex pulse trains (count)


#
# Canonical device types
#
DEVICE_GC100 = 'GC-100'
DEVICE_ITACH = 'iTach'
DEVICE_FLEX = 'iTach Flex'


#
# network ports used
#
GCNET_PORT = 4998			# command port
SERIAL_BASE = 4999			# serial relay ports, sequential

UDP_ADDRESS = '239.255.250.250' # sensor notification multicast address (iTach only)
UDP_PORT = 9132				# default UDP notification port


#
# Exceptions
#
class Error(Exception):
	pass

class DeviceError(Error):
	""" Network device complaint message.

		Carries a numeric code and explanatory text.
		Note that GC-100 and iTach have very different mappings.
	"""
	def __init__(self, code, msg):
		self.code = code
		self.msg = msg

	def __str__(self):
		return self.msg

	def __repr__(self):
		return "<gcnet error %s: %s>" % (self.code, self.msg)
